<?php
/**
 * Handler: for managing Shortcodes
 */
namespace EwTools\Handler;

/**
 * Shortcodes
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\Shortcodes 0.1
 */
class Shortcodes {
	
	/**
	 * Parse a shortcode into an array with optional defaults
	 *
	 * @param string $shortcode
	 * @param array  $defaults
	 *
	 * @uses shortcode_parse_atts()
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function parse( $shortcode, $defaults = [] ) {
		# if shortcode is already registrered, just fetch the key/value pairs and return in array
		$result = shortcode_parse_atts( preg_replace( '/\[|\]/', '', $shortcode ) );
		if ( is_array( $result ) ) {
            array_shift( $result );
			return array_replace( $defaults, $result );
		}
		
		return [];
	}
	
	/**
	 * Render shortcode with optional default values
	 *
	 * @param string $shortcode
	 * @param array  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function render( $shortcode, $default = [] ) {
		echo static::generate( $shortcode, $default );
	}
	
	/**
	 * Generate a string with the output of the shortcode with optional default values
	 *
	 * @param string $shortcode
	 * @param array  $default
	 *
	 * @uses do_shortcode()
	 *
	 * @since 0.1
	 * @return mixed|string
	 */
	public static function generate( $shortcode, $default = [] ) {
		if ( empty( $default ) ) {
			return do_shortcode( $shortcode );
		}
		$tag              = static::parseTag( $shortcode );
		$parsed_shortcode = static::parse( $shortcode, $default );
		
		return do_shortcode( static::build( $tag, $parsed_shortcode ) );
	}
	
	/**
	 * Build a shortcode with a name and properties
	 *
	 * @param string $name
	 * @param array  $properties
	 * @param string $property_wrapper
	 *
	 * @since 0.1
	 * @return string
	 */
	public static function build( $name, $properties = [], $property_wrapper = '"' ) {
		$attributes = "";
		array_walk( $properties, function( $value, $key ) use ( &$attributes, $property_wrapper ) {
			$attributes[] = "{$key}={$property_wrapper}{$value}{$property_wrapper}";
		} );
		
		return "[{$name} " . join( ' ', $attributes ) . "]";
	}
	
	/**
	 * Retrieve the tag/name of the shortcode
	 *
	 * @param $shortcode
	 *
	 * @uses shortcode_parse_atts()
	 *
	 * @since 0.1
	 * @return string
	 */
	public static function parseTag( $shortcode ) {
		$result = shortcode_parse_atts( preg_replace( '/\[|\]/', '', $shortcode ) );
		
		return isset( $result[0] ) ? $result[0] : '';
	}
}