<?php namespace EwTools\Models;

use EwTools\Interfaces\Support\Arrayable;
use EwTools\Interfaces\Support\Jsonable;
use EwTools\Support\Collection;
use EwTools\Support\Fluent;
use EwTools\Support\Str;
use WP_Post;

/**
 * Post
 *
 * @property int id
 * @property array ancestors
 * @property int post_parent
 * @property string post_title
 * 
 * @link    http://infomaker.se
 * @package EwTools\Models
 * @since   EwTools\Models\Post 0.1
 */
class Post extends Fluent {
    
    protected static $type = 'post';
    
    /**
     * Defaults for fetching posts in Wordpress
     *
     * @var array
     */
    protected static $default_options = [
        'sort_order'   => 'ASC',
        'sort_column'  => 'post_title',
        'hierarchical' => true,
        'exclude'      => '',
        'include'      => '',
        'meta_key'     => '',
        'meta_value'   => '',
        'authors'      => '',
        'child_of'     => 0,
        'parent'       => -1,
        'exclude_tree' => '',
        'number'       => '',
        'offset'       => 0,
        'post_status'  => 'publish'
    ];
    
    /**
     * post constructor.
     *
     * @since 0.1
     *
     * @param WP_Post|array $post
     */
    public function __construct( $post = [] ) {
        
        parent::__construct( $this->getArrayableItems( $post ) );
    }
           
    /**
     * Add metadata from a "custom field"/Metabox
     *
     * @param string $key
     * @param mixed  $value
     *
     * @since 0.1
     * @return mixed
     */
    public function addMeta( $key, $value ) {
        if( $this->updateAllowed() ) {
            return add_post_meta( $this->id, $key, json_encode( $value, JSON_UNESCAPED_UNICODE ) );
        }
        
        return false;
    }
    
    /**
     * Retrieve ancestors of the post
     *
     * @since 0.1
     * @return Collection
     */
    public function ancestors() {
        return collect( $this->ancestors )->map( function ( $ancestor ) {
            return static::createFromId( $ancestor );
        } );
    }
    
    /**
     * Retrieve the posts child posts
     *
     * @param array $options
     *
     * @since 0.1
     * @return Collection
     */
    public function children( array $options = [] ) {
        return static::all( array_replace_recursive( [ 'child_of' => $this->id ], $options ) );
    }
    
    /**
     * Retrieve the posts edit url
     *
     * @since 0.1
     * @return false|string
     */
    public function editUrl() {
        return get_edit_post_link( $this->id );
    }
    
    /**
     * Retrieve the data stored on a specific meta key
     *
     * @param string $key
     * @param mixed  $default
     *
     * @since 0.1
     * @return mixed
     */
    public function getMeta( $key = '', $default = [] ) {
        if( Str::isEmpty( $key ) ) {
            return get_post_meta( $this->id, $key, true );
        }
        $meta = json_decode( get_post_meta( $this->id, $key, true ), true );
        
        return $meta ?: $default;
    }
    
    /**
     * Determine if a post has a parent
     *
     * @since 0.1
     * @return bool
     */
    public function hasParent() {
        return (int)$this->post_parent !== 0;
    }
    
    /**
     * Add chars before the name to emphasise the posts hierarchical position to its ancestors.
     *
     * @since 0.1
     * @return string
     */
    public function hierarchicalName() {
        $text      = $this->post_title;
        $ancestors = $this->post_parent > 0 ? count( $this->ancestors() ) : 0;
        
        return Str::prepend( str_repeat( '— ', $ancestors ), $text );
    }
    
    /**
     * Retrieve ancestors of the post
     *
     * @since 0.1
     * @return static
     */
    public function parent() {
        return $this->hasParent() ? static::createFromId( $this->post_parent ) : null;
    }
    
    /**
     * Retrieve the permalink of the post
     *
     * @since 0.1
     * @return false|string
     */
    public function permalink() {
        return static::getPermalink( $this->id );
    }
    
    /**
     * Remove the data stored on a specific meta key
     *
     * @param string $key
     *
     * @since 0.1
     * @return mixed
     */
    public function removeMeta( $key ) {
        if( $this->updateAllowed() ) {
            return delete_post_meta( $this->id, $key );
        }
        
        return false;
    }
    
    /**
     * Return the posts title
     *
     * @param $prefix
     *
     * @since 0.1
     * @return string
     */
    public function title( $prefix = '' ) {
        $title = $this->post_title;
        
        return Str::notEmpty( $prefix ) ? Str::append( $prefix, $title, ' - ' ) : $title;
    }
    
    /**
     * Function to handle if a user can update the post
     *
     * @since 0.1
     * @return bool
     */
    public function updateAllowed() {
        if( isset( $_POST[ 'post_type' ] ) && $_POST[ 'post_type' ] === static::$type ) {
            return current_user_can( 'edit_' . static::$type, $this->id );
        }
        
        return false;
    }
    
    /**
     * Update the value stored on a specific meta key
     *
     * @param string $key
     * @param mixed  $value
     *
     * @since 0.1
     * @return mixed
     */
    public function updateMeta( $key, $value ) {
        if( $this->updateAllowed() ) {
            return update_post_meta( $this->id, $key, json_encode( $value, JSON_UNESCAPED_UNICODE ) );
        }
        
        return false;
    }
    
    /**
     * Results array of items from Collection or Arrayable.
     *
     * @param  mixed $items
     *
     * @return array
     */
    protected function getArrayableItems( $items ) {
        if( is_array( $items ) ) {
            return $items;
        } elseif( $items instanceof WP_Post ) {
            return array_change_key_case( $items->to_array(), CASE_LOWER );
        } elseif( $items instanceof Arrayable ) {
            return $items->toArray();
        } elseif( $items instanceof Jsonable ) {
            return json_decode( $items->toJson(), true );
        }
        
        return (array)$items;
    }
    
    /**
     * Retrieve all posts with optional filtering options
     *
     * @param array $options
     *
     * @uses  get_posts()
     * @since 0.1
     * @return Collection
     */
    public static function all( array $options = [] ) {
        return collect( get_posts( static::setupOptions( $options ) ) )->map( [ static::class, 'createFromPost' ] );
    }
    
    /**
     * Retrieve post by id
     *
     * @param $id
     *
     * @since 0.1
     * @return static
     */
    public static function createFromId( $id ) {
        if( false !== ( $post = get_post( $id ) ) ) {
            return static::createFromPost( $post );
        }
        
        return null;
    }
    
    /**
     * Retrieve post by post
     *
     * @param WP_Post $post
     *
     * @since 0.1
     * @return static
     */
    public static function createFromPost( WP_Post $post ) {
        return new static( $post );
    }
    
    /**
     * Retrieve post by url
     *
     * @param string $url
     *
     * @uses  url_to_postid()
     * @since 0.1
     * @return static
     */
    public static function createFromUrl( $url ) {
        return static::createFromId( url_to_postid( $url ) );
    }
    
    /**
     * Retrieve current post
     *
     * @since 0.1
     * @return static
     */
    public static function current() {
        $post = get_post();
        if( $post ) {
            return static::createFromPost( $post );
        }
        
        return null;
    }
    
    /**
     * Retrieves the full permalink for the current post or post ID.
     *
     * @since 1.0.0
     *
     * @param int|WP_Post $post Optional. Post ID or post object. Default is the global `$post`.
     *
     * @return string|false The permalink URL or false if post does not exist.
     */
    public static function getPermalink( $post ) {
        return get_permalink( $post );
    }
    
    private static function setupOptions( array $options = [] ) {
        return array_replace_recursive( static::$default_options, [ 'post_type' => static::$type ], $options );
    }
}