<?php namespace EwTools\Models;

use EwTools\Interfaces\Support\Selectable;
use EwTools\Storage\CollectionModel;
use EwTools\Support\Collection;
use EwTools\Support\Str;

/**
 * Section
 *
 * @property string $name
 * @property string $slug
 * @property string $class_name
 * @property string $sub_section
 * @link    http://infomaker.se
 * @package EwTools\Models
 * @since   EwTools\Models\Section 0.1
 */
class Section extends CollectionModel implements Selectable {
    
    protected $option_name = 'everyware_registered_sections_by_slug';
    
    protected $fields = [
        'name'        => '',
        'slug'        => '',
        'class_name'  => '',
        'sub_section' => ''
    ];
    
    /**
     * Fetch a collection of Sections that has the specified subsection
     *
     * @param $sub_section
     *
     * @since 0.1
     * @return Collection
     */
    public static function getBySubSection( $sub_section ) {
        return static::all()->filter( function ( self $section ) use ( $sub_section ) {
            return in_array( $sub_section, $section->subSections(), true );
        } );
    }
    
    /**
     * Retrieve the css-class bound to the specified section
     *
     * @param string $key
     * @param string $value
     *
     * @since 0.1
     * @return string
     */
    public static function getClassWhere( $key, $value = '' ) {
        if (Str::isEmpty($value)) {
            return '';
        }

        $section = static::whereIn($key, $value)->first();

        if ( ! $section instanceof self ) {
            return '';
        }

        return $section->class_name;
    }
    
    /**
     * Retrieve the css-class bound to the specified section
     *
     * @param string $name
     *
     * @since 0.1
     * @return string
     */
    public static function getClassByName( $name = '' ) {
        return static::getClassWhere( 'name', $name );
    }
    
    /**
     * Retrieve the css-class bound to the specified section
     *
     * @param string $slug
     *
     * @since 0.1
     * @return string
     */
    public static function getClassBySlug( $slug = '' ) {
        return static::getClassWhere( 'slug', $slug );
    }
    
    /**
     * Retrieve the subsections in the form of an array
     *
     * @since 0.1
     * @return array
     */
    public function subSections() {
        return (array)preg_split( '/\s*(,|\|)\s*/', $this->sub_section );
    }
    
    /**
     * Return data in the form of an array fit for a select-box
     * A preselected value may be sent in.
     *
     * @param mixed $selected
     *
     * @since 0.1
     * @return Collection
     */
    public static function toSelect( $selected = null ) {
        return static::collection()->all()->map( function ( $section ) use ( $selected ) {
            return [
                'value'    => $section[ 'slug' ],
                'text'     => $section[ 'name' ],
                'selected' => $section[ 'slug' ] === $selected
            ];
        } );
    }
    
    protected function onSet( $fields = [] ) {
        if( Str::notEmpty( $fields[ 'name' ] ) ) {
            $fields[ 'slug' ] = Str::slug( $fields[ 'name' ] );
        }
        
        return $fields;
    }
}